#include "pTPM.h"
#include "vTPM.h"
#include "verifier.h"
#include "stdio.h"
#include "stdlib.h"
#include "tss2/tss2_mu.h"
#include <iostream>
#include <iterator>
#include <unordered_map>
#include <inttypes.h>
#include <string>
#include <fstream>
#include <cstring>
using namespace std;

int main()
{
	// Return code for API call
    TSS2_RC rc;

    // Keys that will be used during the attestation process
    //Key primaryKey;
    //Key keyVM1;

    // Start of the project
    cout<<"\n*** vTPM - pTPM binding PoC ***"<<endl;

    //Class initialization
    pTPM pTPM(1);

#ifdef DEBUG
    getchar();
#endif

    vTPM vTPM_1(1);

#ifdef DEBUG
    getchar();
#endif

cout<<"\n*** Definition of the verifier ***"<<endl;

#ifdef DEBUG
    getchar();
#endif

    Verifier verifier(1);

    // Definition of the NV space of type extend
    cout<<"\n*** Definition of the NV space  ***"<<endl;

#ifdef DEBUG
    getchar();
#endif

    pTPM.Allocate_VM_Resources(vTPM_1.getIdentifier());
    pTPM.Allocate_VM_Resources(0); //VMM banks allocation
    
    // Creation of two keys used to sign the quote operation
    cout<<"\n*** Creation of a primary Key for the pTPM ***"<<endl;

#ifdef DEBUG
    getchar();
#endif

    //pTPM.Create_PK(&primaryKey);
    pTPM.CreatePK();

    cout<<"\n*** Creation of the AIK from the primary key ***"<<endl;

#ifdef DEBUG
    getchar();
#endif

    //pTPM.Create_Key(&keyVM1, &primaryKey);
    pTPM.CreateAIK(vTPM_1.getIdentifier());

    //verifier.Load(pTPM.GetRootKeyP(),pTPM.GetAIKP());

    cout<<"\n*** Booting the VM ***"<<endl;

#ifdef DEBUG
    getchar();
#endif
    /* Platform Integrity Measurements
    For testing purpose the first 3 PCRs will simulate the measurements during the Boot process
    The VM during boot time will take the following measurements:
    	- Binaries => PCR 0
    	- Firmware => PCR 1
    	- Boot Loader => PCR 2
	*/

    //vTPM_1.Boot_VM();
    string line;
    TPM2B_MAX_NV_BUFFER dataToExtend;
    TPMI_ALG_HASH hashBank = TPM2_ALG_SHA1;
   	int i = 0;

   	fstream vmmfile;
    vmmfile.open("VMMlogs.log");
	i = 0;
	while(!vmmfile.eof())
    {

    	int pcrNumber = i/50;

        getline(vmmfile,line);
        memcpy(dataToExtend.buffer,line.c_str(),line.size());
        cout << line << endl;

        // For each measurement taken we aggregate the information inside the corresponding VM
        pTPM.Extend_nvPCR(0, pcrNumber, hashBank, &dataToExtend);
        i++;
    }


    // simulating the booting process of the VM
    fstream rfile;
    rfile.open("VMlogs.log");
    i = 0;
	while(!rfile.eof())
    {

    	int pcrNumber = i/50;

        getline(rfile,line);
        memcpy(dataToExtend.buffer,line.c_str(),line.size());
        cout << line << endl;
        dataToExtend.size = line.size();
        cout << dataToExtend.buffer << endl;

        // For each measurement taken we aggregate the information inside the corresponding VM
        pTPM.Extend_nvPCR(vTPM_1.getIdentifier(), pcrNumber, hashBank, &dataToExtend);
        i++;
    }

    cout<<"\n*** Creating the Integrity Request ***"<<endl;

#ifdef DEBUG
    getchar();
#endif
    IntegrityRequest integrityRequest;
    integrityRequest = verifier.CreateIntegrityRequest();

    cout<<"\n*** Sending the Integrity Request to the Agent ***"<<endl;

#ifdef DEBUG
    getchar();
#endif
    IntegrityReport integrityReport;
    integrityReport = pTPM.IntegrityRequestFrom_vTPM(vTPM_1.getIdentifier(),integrityRequest);

    cout<<"\n*** Processing the Integrity Response received from the Agent ***"<<endl;

#ifdef DEBUG
    getchar();
#endif

    verifier.VerifyIntegrityReport(integrityReport, pTPM.GetRootKey(), pTPM.GetAIK());


    // Release of Resources
    cout<<"\n*** Finalization, end of test TPM ***"<<endl;
#ifdef DEBUG
    getchar();
#endif

    pTPM.Release_VM_Resources(vTPM_1.getIdentifier());
    pTPM.Release_VM_Resources(0);
	return 0;
}