#ifndef pTPM_H_INCLUDED
#define pTPM_H_INCLUDED

#include "tss2/tss2_tcti_mssim.h"
#include "tss2/tss2_esys.h"
#include "tss2/tss2_tpm2_types.h"
#include "structure.h"
#include <unordered_map>

#include "MapKeyClass.h"

using namespace std;

class pTPM
{
    public:
        // Constructor
        pTPM(int number);

        // Destructor
        ~pTPM();

        // Function for Resource Allocation
        void Allocate_VM_Resources(UINT32 vID);

        // Function for Resource release
        void Release_VM_Resources(UINT32 vID);

        // Function for the creation of the primary key associated with th pTPM
        void CreatePK();

        // Function for the creation of a derived key associated with a vTPM
        void Create_Key(Key* derivedKey, Key* primaryKey);

        // Function for the creation of a derived key associated with a vTPM
        void CreateAIK(UINT32 vID);

        // Function for the extension of the space
        void Extend_nvPCR(UINT32 vID, int pcrNumber, TPMI_ALG_HASH hashBank, TPM2B_MAX_NV_BUFFER* dataToExtend);

        // Function for Handling the Integrity Request received from the vTPM
        IntegrityReport IntegrityRequestFrom_vTPM(UINT32 vID, IntegrityRequest integrityRequest);

        //Function to find the memory allocation of a PCR
        ESYS_TR GetIndexFromMap(UINT32 identifier, TPMI_ALG_HASH hash, UINT32 pcrNumber);

        //Export public Key
        Key GetRootKey();
        Key GetAIK();

        //Export public Key
        Key* GetRootKeyP();
        Key* GetAIKP();

    private:
        // Method to initialize the TCTI context
        void Init_Tcti_Tabrmd_Context();

        // Method to finalize the TCTI context
        void Finalize_Tcti_Tabrmd_Context();

        // Methods to initialize the ESYS context
        void Init_Esys_Context();

        // Method to finalize the ESYS context
        void Finalize_Esys_Context();

        // Error handling function
        int ErrorHandling(TSS2_RC rc);

        // Function to extract hash size 
        UINT16 GetHashSize(TPMI_ALG_HASH hash);

        // Function for nv-index definition
        ESYS_TR Define_nvPCR (TPMI_ALG_HASH hashBank, TPMI_RH_NV_INDEX freeIndex);

        // Function for undefining an NV space
        void Undefine_NV_Space(ESYS_TR* handle);

        // Function for Updating the MAP
        void UpdateMap(unordered_map<MapKey,ESYS_TR>* mapNVarea, UINT32 identifier, TPMI_ALG_HASH hash, UINT32 pcrNumber, ESYS_TR nvIndex);


        // New command for deep attestation
        /*
			vID [in]
			pcrSelection [in]
			nonce [in]
			inScheme [in]
			signingKey [in]
			primaryKey [in]
			attestStructure [out]
			signatureStructure [out]
			pcrValues [out]
        */
        void TPM2_VIRT_QUOTE(UINT32 vID, TPML_PCR_SELECTION pcrSelection, TPM2B_DATA nonce, TPMT_SIG_SCHEME inScheme, Key* signingKey, Key* primaryKey, TPM2B_ATTEST* attestStructure, TPMT_SIGNATURE* signatureStructure, TPML_DIGEST* pcrValues);

    private:
        // Pointers to contexts
        TSS2_TCTI_CONTEXT *tcti_context = nullptr;
        ESYS_CONTEXT      *esys_context = nullptr;

        // Keys that will be used during the attestation process
    	Key primaryKey;
    	Key keyVM1;

        // Identifier variables
        int identifier;

        // nvIndex store the last index defined in nv space
        TPMI_RH_NV_INDEX freeIndex;

        // Definition of a map to store the association between vTPM - PCR
        /*
            MAP:
            Key = < #v-TPM , Hash-Alg, #-pcrNumber > 
            value = ESYS_TR nvHandle
        */
        unordered_map<MapKey, ESYS_TR> mapNVarea;
};

#endif