#ifndef vTPM_H_INCLUDED
#define vTPM_H_INCLUDED

#include "tss2/tss2_esys.h"
#include "structure.h"

class vTPM
{
private:
    // This is a unique id 
    UINT32 identifier;

    // nvHandle
    ESYS_TR nvHandle;

    // Session and authorization 
    TPM2B_NONCE nonceCaller;
    TPM2B_AUTH authValue;

    // Pointer to the esys_context
    ESYS_CONTEXT* esys_context = nullptr;
    TSS2_TCTI_CONTEXT *tcti_context = nullptr;

private:
        // Method to initialize the TCTI context
        void Init_Tcti_Tabrmd_Context();

        // Method to finalize the TCTI context
        void Finalize_Tcti_Tabrmd_Context();

        // Methods to initialize the ESYS context
        void Init_Esys_Context();

        // Method to finalize the ESYS context
        void Finalize_Esys_Context();

        // Error handling function
        int ErrorHandling(TSS2_RC rc);

        // Function to extract hash size 
        UINT16 GetHashSize(TPMI_ALG_HASH hash);
    
public:
    // Constructor
    vTPM(UINT32 id);

    // Destructor
    ~vTPM();

    // Setter for nvHandle
    void setNvHandle(ESYS_TR handle);

    // Getter for nvHandle Pointer
    ESYS_TR* getNvHandle();

    // Getter for the identifier
    UINT32 getIdentifier();

    // Function to load an external key 
    ESYS_TR Load_Public_Key(TPM2B_PUBLIC publicData);

    // Function for the creation of the primary key associated with th vTPM - used for session authentication 
    void Create_PK(Key* key);

    void Test(TPM2B_PUBLIC inPublic);

};

#endif